defmodule Slax.Chat.Room do
  use Ecto.Schema
  import Ecto.Changeset
  alias Slax.Accounts.User
  alias Slax.Chat.Message
  alias Slax.Chat.RoomMembership

  schema "rooms" do
    field :name, :string
    field :topic, :string
    field :unread_message_count, :integer, virtual: true, default: 0

    many_to_many :members, User, join_through: RoomMembership

    has_many :memberships, RoomMembership
    has_many :messages, Message

    timestamps(type: :utc_datetime)
  end

  @doc false
  def changeset(room, attrs) do
    room
    |> cast(attrs, [:name, :topic])
    |> validate_required([:name])
    |> validate_length(:name, max: 80)
    |> validate_format(:name, ~r/\A[a-z0-9-]+\z/,
      message: "can only contain lowercase letters, numbers, and dashes"
    )
    |> validate_length(:topic, max: 200)
    |> unique_constraint(:name)
  end
end
